"""
Models and Enums for the database.
All table classes should be suffixed with `Model`.
"""

from enum import Enum, auto

from sqlalchemy import Column, Integer, String, DateTime, BigInteger
from sqlalchemy.sql import func
from sqlalchemy.orm import relationship
from sqlalchemy.ext.declarative import declarative_base

Base = declarative_base()


class AuditModel(Base):
    """
    Table for taking audits.
    """

    __tablename__ = "audit"

    id = Column(Integer, primary_key=True, autoincrement=True)
    discord_user_id = Column(BigInteger, nullable=False)
    message = Column(String, nullable=False)
    created = Column(DateTime(timezone=True), server_default=func.now(), nullable=False)
    active = Column(Integer, default=True, nullable=False)


class SentArticleModel(Base):
    """
    Table for tracking articles that have been served by the bot.
    """

    __tablename__ = "sent_articles"

    id = Column(Integer, primary_key=True, autoincrement=True)
    discord_message_id = Column(BigInteger, nullable=False)
    discord_channel_id = Column(BigInteger, nullable=False)
    discord_server_id = Column(BigInteger, nullable=False)
    article_url = Column(String, nullable=False)
    active = Column(Integer, default=True, nullable=False)


class RssSourceModel(Base):
    """
    Table for user submitted news feeds.
    """

    __tablename__ = "rss_source"

    id = Column(Integer, primary_key=True, autoincrement=True)
    discord_server_id = Column(BigInteger, nullable=False)
    rss_url = Column(String, nullable=False)
    active = Column(Integer, default=True, nullable=False)


class FeedChannelModel(Base):
    """
    Table representing discord channels to be used for news feeds.
    """

    __tablename__ = "feed_channel"

    id = Column(Integer, primary_key=True, autoincrement=True)
    discord_channel_id = Column(BigInteger, nullable=False)
    active = Column(Integer, default=True, nullable=False)
